#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2014 Adrian Perez <aperez@igalia.com>
#
# Distributed under terms of the MIT license.

import unittest
import wcfg
from os import path
from os import listdir


class TestConfigFiles(unittest.TestCase):
    def check_file(self, filepath):
        f = open(filepath, "rb")
        value = wcfg.load(f)
        f.close()
        self.assertTrue(isinstance(value, dict))

    @classmethod
    def setup_tests(cls):
        dirpath = path.abspath(path.dirname(__file__))
        for filename in listdir(dirpath):
            if filename.endswith(".conf"):
                sanitized_name = "test_" + filename[:-5].replace("-", "_")
                filepath = path.join(dirpath, filename)
                setattr(cls, sanitized_name,
                        lambda self: self.check_file(filepath))


TestConfigFiles.setup_tests()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2014 Adrian Perez <aperez@igalia.com>
#
# Distributed under terms of the GPL3 license or, if that suits you
# better the MIT/X11 license.

import unittest
import wcfg
import six
from textwrap import dedent


class TestParser(unittest.TestCase):

    @staticmethod
    def parser(string):
        return wcfg.Parser(six.BytesIO(string.encode("utf-8")))

    def test_parse_valid_booleans(self):
        booleans = (
            (u"True", True),
            (u"true", True),
            (u"False", False),
            (u"false", False),
        )
        for item, expected in booleans:
            value = self.parser(item).parse_bool()
            self.assertEquals(expected, value)
            self.assertTrue(isinstance(value, bool))

    def check_numbers(self, numbers, type_):
        for item, result in numbers:
            value = self.parser(item).parse_number()
            self.assertEquals(result, value)
            self.assertTrue(isinstance(value, type_))
            # Trailing whitespace must not alter the result.
            value = self.parser(item + " ").parse_number()
            self.assertEquals(result, value)
            self.assertTrue(isinstance(value, type_))

    def test_parse_valid_integer_numbers(self):
        numbers = (
            (u"1", 1),
            (u"+1", 1),
            (u"-2", -2),
            (u"123", 123),
            (u"+123", 123),
            (u"-123", -123),
        )
        self.check_numbers(numbers, six.integer_types)

    def test_parse_valid_hex_numbers(self):
        numbers = (
            (u"0x1", 1),
            (u"0X1", 1),
            (u"0xcafe", 0xCAFE),
            (u"0XCAFE", 0xCAFE),
            (u"0xCAFE", 0xCAFE),
            (u"0xCaFe", 0xCAFE),
            (u"0x1234", 0x1234),
            (u"0xC00FFEE", 0xC00FFEE),
            (u"0xdeadbeef", 0xdeadbeef),
        )
        self.check_numbers(numbers, six.integer_types)

    def test_parse_valid_octal_numbers(self):
        numbers = (
            (u"01", 0o1),
            (u"01234567", 0o1234567),
            (u"042", 0o42),
        )
        self.check_numbers(numbers, six.integer_types)

    def test_parse_valid_float_numbers(self):
        numbers = (
            (u"1.", 1.0),
            (u".5", 0.5),
            (u"1.5", 1.5),
            (u"1e3", 1e3),
            (u"1e-3", 1e-3),
            (u"1.e3", 1e3),
            (u"1.e-3", 1e-3),
            (u".5e3", 0.5e3),
            (u".5e-3", 0.5e-3),
            (u"1.5e3", 1.5e3),
            (u"1.5e-3", 1.5e-3),
        )
        def gen_signs():
            for item, value in numbers:
                yield (item, value)
                yield (u"+" + item, value)
                yield (u"-" + item, -value)

        self.check_numbers(gen_signs(), float)

    def test_parse_valid_identifiers(self):
        identifiers = (
            # Typical definition of identifiers.
            u"foo",
            u"ident-with-dashes",
            u"-leading-dash",
            u"trailing-dash-",
            u"ident_with_underscores",
            u"_leading_underscore",
            u"trailing_underscore_",
            u"ident.with.dots",
            # Unicode identifiers must work.
            u"☺",
            u"武術",
            u"空手",
            u"→",
            # Mixed.
            u"arrow→",
            u"Trømso",
            u"Güedángaños",
        )
        for item in identifiers:
            ident = self.parser(item).parse_identifier()
            self.assertEquals(item, ident)
            self.assertTrue(isinstance(ident, six.text_type))

    def check_strings(self, strings, type_):
        for item in strings:
            if isinstance(item, tuple):
                item, expected = item
            else:
                expected = item
            value = self.parser(u"\"" + item + u"\"").parse_string()
            self.assertEquals(expected, value)
            self.assertTrue(isinstance(value, type_))

    def test_parse_valid_strings(self):
        strings = (
            u"",
            u"this is a string",
            u" another with leading space",
            u"yet one more with trailing space ",
            u"unicode: this → that, Trømso, Java™, ☺",
            (u"escaped backslash: \\\\", u"escaped backslash: \\"),
            (u"escaped double quote: \\\"", u"escaped double quote: \""),
            (u"escaped UTF-8: \\☺", u"escaped UTF-8: ☺"),
        )
        self.check_strings(strings, six.text_type)

    def test_parse_valid_arrays(self):
        arrays = (
            (u"[]", []),
            (u"[ ]", []),
            (u"[1]", [1]),
            (u"[ 1]", [1]),
            (u"[1 ]", [1]),
            (u"[ 1 ]", [1]),
            (u"[1 2]", [1, 2]),
            (u"[ 1 2]", [1, 2]),
            (u"[1 2 ]", [1, 2]),
            (u"[ 1 2 ]", [1, 2]),
        )
        for item, expected in arrays:
            value = self.parser(item).parse_value()
            self.assertTrue(isinstance(value, list))
            self.assertListEqual(expected, value)
            # Replacing the spaces with newlines should work as well.
            value = self.parser(item.replace(" ", "\n")).parse_value()
            self.assertTrue(isinstance(value, list))
            self.assertListEqual(expected, value)

    def test_parse_valid_arrays_with_commas(self):
        arrays = (
            (u"[1,]", [1]),        # Dangling commas are allowed.
            (u"[1,2]", [1, 2]),    # Spaces around commas are optional.
            (u"[1, 2]", [1, 2]),   # Space can be provided, of course...
            (u"[1 , 2]", [1, 2]),  # ...even on both sides
            (u"[1\n,2]", [1, 2]),  # Newlines and commas.
            (u"[1,\n2]", [1, 2]),  # Ditto.
            (u"[1\t,2]", [1, 2]),  # Tabs around commas.
            (u"[1,2,]", [1, 2]),   # More than one item and dangling comma.
            (u"[1, 2, ]", [1, 2]), # Spaces after dangling comma.
            (u"[1 2,3]", [1,2,3]), # Mixed spaces and commas.
        )
        for item, expected in arrays:
            value = self.parser(item).parse_value()
            self.assertTrue(isinstance(value, list))
            self.assertListEqual(expected, value)

    def test_parse_invalid_arrays(self):
        invalid_arrays = (
            u"[",      # Unterminated array.
            u"[1",     # Ditto.
            u"[1 2",   # Ditto.
            u"[[]",    # Unterminated inner array.
            u"(1)",    # Not an array.
            u"[:]",    # Invalid value inside array.
            u"[\"]",   # Unterminated string inside array.
            u"[\"]\"", # Unbalanced double-quote and brackets.
        )
        for item in invalid_arrays:
            with self.assertRaises(wcfg.ParseError):
                self.parser(item).parse_value()

    def test_parse_invalid_arrays_with_commas(self):
        invalid_arrays = (
            u"[,]",     # Array with holes.
            u"[ ,]",    # Ditto.
            u"[, ]",    # Ditto.
            u"[,,]",    # Multiple holes.
            u"[\v,\t]", # Ditto.
            u"[,1]",    # Leading comma.
            u"[1,,]",   # Double trailing comma.
        )
        for item in invalid_arrays:
            with self.assertRaises(wcfg.ParseError):
                self.parser(item).parse_value()

    def test_parse_invalid_numbers(self):
        invalid_numbers = (
            u"", u"-", u"+", u"-+", u"a", u"☺", u"-.", u".", u"e", u".e",
            u"+e", u"-e", u"-.e", u"+.e", u"e+", u"e-", u".-e", u".+e",
            u"--", u"++", u"+1e3.", u"..1", u"1.2.", u"1..2", u"\"foo\"",
            u"True", u"False", u"{}", u"[]", u"()", u"0xx00", "0.1AeA3",
            "-0x3", "-032", u"ee", u"1ee", u"1e1e1", u"0.1x2", u"1x.0",
        )
        for item in invalid_numbers:
            with self.assertRaises(wcfg.ParseError):
                self.parser(item).parse_number()

    def test_parse_invalid_booleans(self):
        invalid_booleans = (
            u"Tr", u"TRUE", u"TrUe", u"TruE", u"TrUE",
            u"Fa", u"FALSE", u"FaLSE", u"FaLsE", u"FalsE",
            u"1", u"0", u"\"True\"", u"\"False\"",
        )
        for item in invalid_booleans:
            with self.assertRaises(wcfg.ParseError):
                self.parser(item).parse_bool()

    def test_parse_invalid_strings(self):
        invalid_strings = (
            u"\"",     # Unterminated.
            u"\"a",    # Ditto.
            u"\"\\\"", # Ditto.
        )
        for item in invalid_strings:
            with self.assertRaises(wcfg.ParseError):
                self.parser(item).parse_string()


class TestDump(unittest.TestCase):

    @staticmethod
    def dump_value(value):
        stream = six.BytesIO()
        wcfg._dump_value(value, stream, 0)
        return stream.getvalue()

    def test_dump_values(self):
        values = (
            (True, b"True"),
            (False, b"False"),
            (123, b"123"),
            (-123, b"-123"),
            (0xFF, b"255"),  # An hex literal in Python is still an integer.
            (0o5, b"5"),     # Ditto for octals in Python.
            (0.5, b"0.5"),
            (-0.5, b"-0.5"),
            (six.b("byte string"), b'"byte string"'),
            (u"a string", b'"a string"'),
            (u"double quote: \"", b'"double quote: \\""'),
            ((1, 2, 3), b"[\n  1\n  2\n  3\n]"),
            ([1, 2, 3], b"[\n  1\n  2\n  3\n]"),
            ({"a": 1, "b": 2}, b"{\n  a: 1\n  b: 2\n}"),
        )
        for value, expected in values:
            result = self.dump_value(value)
            self.assertEquals(expected, result)
            self.assertTrue(isinstance(result, bytes))

    def test_invalid_key_types(self):
        invalid_keys = (
            42, 3.14,    # Numeric.
            object(),    # Objects.
            object,      # Classes.
            True, False, # Booleans.
            [], (), {},  # Collections.
        )
        for key in invalid_keys:
            with self.assertRaises(TypeError):
                wcfg.dumps({ key: True })

    def test_invalid_key_values(self):
        invalid_keys = (
            "col:on",    # Colons are not allowed.
            "sp ace",    # Ditto for spaces.
        )
        for key in invalid_keys:
            with self.assertRaises(ValueError):
                wcfg.dumps({ key: True })

    def test_dump_non_dict(self):
        invalid_values = (
            "string",
            42, 3.14,    # Numbers.
            object(),    # Objects.
            object,      # Classes.
            True, False, # Booleans.
            [], (),      # Non-dict collections.
        )
        for value in invalid_values:
            with self.assertRaises(TypeError):
                wcfg.dumps(value)

    def test_invalid_values(self):
        invalid_values = (
            object(),    # Objects.
            object,      # Classes.
        )
        for value in invalid_values:
            with self.assertRaises(TypeError):
                wcfg.dumps({ "value": value })


class TestAPI(unittest.TestCase):

    TEST_VALUES = (
        ({}, ""),
        ({"a": 1, "b": 2}, """\
            a: 1
            b: 2
            """),
        ({"a": {"b": {}}}, """\
            a: {
              b: {
              }
            }
            """),
        ({"a": {"t": True}, "nums": [1e4, -2, 0xA], "f": False}, """\
            a: {
              t: True
            }
            f: False
            nums: [
              10000.0
              -2
              10
            ]
            """),
        ({"a": [{"b": 1}, [1, 2, 3]]}, """\
            a: [
              {
                b: 1
              }
              [
                1
                2
                3
              ]
            ]
            """),
    )

    TEST_VALUES_LOADS_ONLY = (
        ({}, "# This only has a comment"),
        ({"list": [1, 2, 3]}, "list:[1 2 3]"),
        ({"list": [1, 2, 3]}, "list: [ 1 2 3 ]"),
        ({"list": [1, 2, 3]}, """\
            # Leading comment
            list: [1 2 3] # Inline comment
            # Trailing comment
            """),
        ({"list": [1, 2, 3]}, """\
            # This skips the optional colon
            list [
                1 2 3
            ]"""),
    )

    def get_dumps_test_values(self):
        for item in self.TEST_VALUES:
            yield item

    def get_loads_test_values(self):
        for item in self.TEST_VALUES:
            yield item
        for item in self.TEST_VALUES_LOADS_ONLY:
            yield item

    def test_dumps(self):
        for value, expected in self.get_dumps_test_values():
            result = wcfg.dumps(value)
            expected = six.b(dedent(expected))
            self.assertEquals(expected, result)
            self.assertTrue(isinstance(result, bytes))

    def test_loads(self):
        for expected, value in self.get_loads_test_values():
            result = wcfg.loads(six.b(dedent(value)))
            self.assertTrue(isinstance(result, dict))
            self.assertDictEqual(expected, result)
            # Passing Unicode text should work as well.
            result = wcfg.loads(dedent(value))
            self.assertTrue(isinstance(result, dict))
            self.assertDictEqual(expected, result)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2014 Adrian Perez <aperez@igalia.com>
#
# Distributed under terms of the GPL3 license or, if that suits you
# better the MIT/X11 license.

"""This module provide a pythonic way to parse configuration files
used in libwheel
"""

__version__ = 3

import six
import string

_SPACE = six.b(" ")
_COMMA = six.b(",")
_COLON = six.b(":")
_LBRACE = six.b("{")
_RBRACE = six.b("}")
_NEWLINE = six.b("\n")
_LBRACKET = six.b("[")
_RBRACKET = six.b("]")
_DQUOTE = six.b("\"")
_OCTOTHORPE = six.b("#")
_SLASHDQUOTE = six.b("\\\"")
_BACKSLASH = six.b("\\")
_NUMBER_SIGNS = six.b("+-")
_ZERO = six.b("0")
_HEX_X = six.b("xX")
_NUMBER_EXP = six.b("eE")
_DOT = six.b(".")
_EOF = six.b("")
_BOOL_LEADERS = six.b("tTfF")
_TRUE_T = six.b("tT")
_TRUE_RUE = (six.b("r"), six.b("u"), six.b("e"))
_FALSE_F = six.b("fF")
_FALSE_ALSE = (six.b("a"), six.b("l"), six.b("s"), six.b("e"))
_TRUE = six.b("True")
_FALSE = six.b("False")


whitespaces = six.b(string.whitespace)
_HEX_CHARS = six.b("abcdefABCDEF")
_NUMBER_CHARS = six.b(string.digits) + \
        _HEX_CHARS + _DOT + _NUMBER_EXP + _NUMBER_SIGNS + _HEX_X


def _dump_value(value, stream, indent):
    if isinstance(value, float):
        stream.write(str(value).encode("ascii"))
    elif isinstance(value, bool):
        stream.write(_TRUE if value else _FALSE)
    elif isinstance(value, six.integer_types):
        stream.write(str(value).encode("ascii"))
    elif isinstance(value, six.text_type):
        stream.write(_DQUOTE)
        stream.write(value.encode("utf-8").replace(_DQUOTE, _SLASHDQUOTE))
        stream.write(_DQUOTE)
    elif isinstance(value, six.string_types) or isinstance(value, bytes):
        stream.write(_DQUOTE)
        stream.write(value.replace(_DQUOTE, _SLASHDQUOTE))
        stream.write(_DQUOTE)
    elif isinstance(value, (tuple, list)):
        stream.write(_LBRACKET)
        for item in value:
            stream.write(_NEWLINE)
            stream.write(_SPACE * ((indent + 1) * 2))
            _dump_value(item, stream, indent + 1)
        stream.write(_NEWLINE)
        stream.write(_SPACE * (indent * 2))
        stream.write(_RBRACKET)
    elif isinstance(value, dict):
        stream.write(_LBRACE)
        stream.write(_NEWLINE)
        _dump_dict(value, stream, indent + 1)
        stream.write(_SPACE * (indent * 2))
        stream.write(_RBRACE)
    else:
        raise TypeError("Values of type " + str(type(value)) +
                        " cannot be dumped")


def _dump_dict(value, stream, indent):
    # Dictionaries are always dumped with their keys sorted,
    # in order to produce a predictible output.
    for k in sorted(six.iterkeys(value)):
        v = value[k]
        if isinstance(k, six.text_type):
            k = k.encode("utf-8")
        elif not isinstance(k, six.string_types):
            raise TypeError("Key is not a string: " + repr(k))
        if _COLON in k:
            raise ValueError("Key contains a colon: " + repr(k))
        if _SPACE in k:
            raise ValueError("Key contains a space: " + repr(k))
        stream.write(_SPACE * (indent * 2))
        stream.write(k)
        stream.write(_COLON)
        stream.write(_SPACE)
        _dump_value(v, stream, indent)
        stream.write(_NEWLINE)


def dump(value, stream):
    if not isinstance(value, dict):
        raise TypeError("Dictionary value expected")
    _dump_dict(value, stream, 0)


def dumps(value):
    output = six.BytesIO()
    dump(value, output)
    return output.getvalue()


class ParseError(ValueError):
    def __init__(self, line, column, message):
        super(ParseError, self).__init__(str(line) + ":" + str(column) +
                                         ": " + message)
        self.line = line
        self.column = column
        self.message = message


class Parser(object):
    def __init__(self, stream):
        self.look = None
        self.line = 1
        self.column = 0
        self.stream = stream
        self.getchar()

    def error(self, message):
        raise ParseError(self.line, self.column, message)

    def _basic_match(self, char, expected_message):
        if self.look != char:
            if expected_message is None:  # pragma: no cover
                expected_message = "character '" + str(char) + "'"
            self.error("Unexpected input '" + str(self.look) + "', " +
                    str(expected_message) + " was expected")
        self.getchar()

    def match(self, char, expected_message=None):
        self._basic_match(char, expected_message)
        self.skip_whitespace()

    def match_in(self, chars, expected_message=None):  # pragma: no cover
        if self.look not in chars:
            if expected_message is None:
                expected_message = "one of '" + str(chars) + "'"
            self.error("Unexpected input '" + str(self.look) + "', " +
                    str(expected_message) + " was expected")
        self.getchar()
        self.skip_whitespace()

    def match_sequence(self, chars, expected_message=None):
        for char in chars:
            self._basic_match(char, expected_message)
        self.skip_whitespace()

    def getchar(self):
        self.look = self.stream.read(1)
        if self.look == _EOF:
            return
        if self.look == _NEWLINE:
            self.column = 0
            self.line += 1
        self.column += 1
        # Skip over comments
        while self.look == _OCTOTHORPE:
            while self.look != _NEWLINE and self.look != _EOF:
                self.look = self.stream.read(1)
            self.column = 1
            self.line += 1
        # self.getchar()

    def skip_whitespace(self):
        while self.look != _EOF and self.look in whitespaces:
            self.getchar()

    def parse_identifier(self):
        identifier = six.BytesIO()
        while self.look != _COLON and self.look not in whitespaces:
            identifier.write(self.look)
            self.getchar()
        self.skip_whitespace()
        return identifier.getvalue().decode("utf-8")

    def parse_bool(self):
        if self.look in _TRUE_T:
            remaining = _TRUE_RUE
            ret = True
        elif self.look in _FALSE_F:
            remaining = _FALSE_ALSE
            ret = False
        else:
            self.error("True or False expected for boolean")
        self.getchar()
        self.match_sequence(remaining, _TRUE if ret else _FALSE)
        return ret

    def parse_string(self):
        value = six.BytesIO()
        char = self.stream.read(1)
        while char != _DQUOTE and char != _EOF:
            if char == _BACKSLASH:
                # A backslash picks the next character literally.
                char = self.stream.read(1)
            value.write(char)
            char = self.stream.read(1)
        if char != _DQUOTE:
            self.error("Unterminated string")
        self.getchar()
        self.skip_whitespace()
        return value.getvalue().decode("utf-8")

    def parse_number(self):
        number = six.BytesIO()

        # Optional sign.
        has_sign = False
        if self.look in _NUMBER_SIGNS:
            has_sign = True
            number.write(self.look)
            self.getchar()

        # Detect octal and hexadecimal numbers.
        is_hex = False
        is_octal = False
        if self.look == _ZERO:
            number.write(self.look)
            self.getchar()
            if self.look in _HEX_X:
                is_hex = True
                number.write(self.look)
                self.getchar()
            else:
                is_octal = True

        # Read the rest of the number.
        dot_seen = False
        exp_seen = False
        while self.look != _EOF and self.look in _NUMBER_CHARS:
            if self.look in _NUMBER_EXP and not is_hex:
                if exp_seen:
                    self.error("Malformed number at '" + str(self.look) + "'")
                exp_seen = True
                # Handle the optional sign of the exponent.
                number.write(self.look)
                self.getchar()
                if self.look in _NUMBER_SIGNS:
                    number.write(self.look)
                    self.getchar()
            else:
                if self.look == _DOT:
                    if dot_seen:
                        self.error("Malformed number at '" +
                                str(self.look) + "'")
                    dot_seen = True
                number.write(self.look)
                self.getchar()
        self.skip_whitespace()

        # Return number converted to the most appropriate type.
        number = number.getvalue().decode("ascii")
        try:
            if is_hex:
                assert not is_octal
                if exp_seen or dot_seen or has_sign:
                    raise ValueError(str(number))
                return int(number, 16)
            elif is_octal:
                assert not is_hex
                if dot_seen or exp_seen or has_sign:
                    raise ValueError(str(number))
                return int(number, 8)
            elif dot_seen or exp_seen:
                assert not is_hex
                assert not is_octal
                return float(number)
            else:
                assert not is_hex
                assert not is_octal
                assert not exp_seen
                assert not dot_seen
                return int(number)
        except ValueError:
            self.error("Malformed number: '" + str(number) + "'")

    def parse_value(self):
        value = None
        if self.look == _DQUOTE:
            value = self.parse_string()
        elif self.look == _LBRACE:
            self.match(_LBRACE)
            value = self.parse_keyval_items()
            self.match(_RBRACE)
        elif self.look == _LBRACKET:
            self.match(_LBRACKET)
            value = self.parse_array_items()
            self.match(_RBRACKET)
        elif self.look in _BOOL_LEADERS:
            value = self.parse_bool()
        else:
            value = self.parse_number()
        return value

    def parse_array_items(self):
        result = []
        while self.look != _EOF and self.look != _RBRACKET:
            result.append(self.parse_value())
            self.skip_whitespace()
            if self.look == _COMMA:
                self.getchar()
                self.skip_whitespace()
        return result

    def parse_keyval_items(self):
        result = {}
        self.skip_whitespace()
        while self.look != _EOF and self.look != _RBRACE:
            key = self.parse_identifier()
            if self.look == _COLON:
                self.match(_COLON)
            self.skip_whitespace()
            result[key] = self.parse_value()
        return result


def load(stream):
    return Parser(stream).parse_keyval_items()


def loads(bytestring):
    if isinstance(bytestring, six.text_type):
        bytestring = bytestring.encode("utf-8")
    return load(six.BytesIO(bytestring))



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2014 Adrian Perez <aperez@igalia.com>
#
# Distributed under terms of the GPL3 license or, if that suits you
# better the MIT/X11 license.

from setuptools import setup
from os import path


def distrib_file(*relpath):
    try:
        return open(path.join(path.dirname(__file__), *relpath), "rU")
    except:
        class DummyFile(object):
            read = lambda self: ""
        return DummyFile()


def wcfg_version():
    for line in distrib_file("wcfg.py"):
        if line.startswith("__version__"):
            line = line.split()
            if line[0] == "__version__":
                return line[2]
    return None


def wcfg_readme():
    return distrib_file("README.md").read()


setup(
    name="wcfg",
    version=wcfg_version(),
    description="Parser for hyerarchical text data and configuration files",
    long_description=wcfg_readme(),
    author="Adrian Perez de Castro",
    author_email="aperez@igalia.com",
    url="https://github.com/aperezdc/python-wcfg",
    py_modules=["wcfg"],
    install_requires=["six>=1.8.0"],
    license="Dual GPL3 / MIT",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python",
        "Operating System :: OS Independent",
    ])
